/**************************************************************************/
/*  gfaz.c Version 3.0 Modulo de parchis.c                                */
/*  Copyright (C) 1992, 1999 Mariano Alvarez Fernandez                    */
/*  Lea en el fichero LEEME.TXT los detalles                              */
/*  See README.TXT for details                                            */
/**************************************************************************/
/*  Rutinas de la interfaz grafica                                        */
/**************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <mgrx.h>
#include <mgrxkeys.h>
#include "grsound.h"
#include "gfaz.h"

GrColor colores[16];
static int hay_raton = 0;
static int cuenta_puntero = 0;

static GrFont * grf_texto;

static struct AreaRaton *area;
static int num_areas = 0;
static int xorg = 0, yorg = 0;
static void (*fespera)( int,int ) = NULL;

/**************************************************************************/

int gfaz_ini( void )
{
//  __djgpp_set_ctrl_c( 0 );
//  _go32_want_ctrl_break( 1 );
//  GrSetDriver( "stdvga gw 640 gh 480 nc 16" );
  GrSetMode( GR_default_graphics );
  colores[0]  = GrAllocColor( 0,   0,   0   );     /* black */
  colores[1]  = GrAllocColor( 0,   0,   170 );     /* blue */
  colores[2]  = GrAllocColor( 0,   170, 0   );     /* green */
  colores[3]  = GrAllocColor( 0,   170, 170 );     /* cyan */
  colores[4]  = GrAllocColor( 170, 0,   0   );     /* red */
  colores[5]  = GrAllocColor( 170, 0,   170 );     /* magenta */
  colores[6]  = GrAllocColor( 170, 85,  0   );     /* brown */
  colores[7]  = GrAllocColor( 170, 170, 170 );     /* light gray */
  colores[8]  = GrAllocColor( 85,  85,  85  );     /* dark gray */
  colores[9]  = GrAllocColor( 85,  85,  255 );     /* light blue */
  colores[10] = GrAllocColor( 85,  255, 85  );     /* light green */
  colores[11] = GrAllocColor( 85,  255, 255 );     /* light cyan */
  colores[12] = GrAllocColor( 255, 85,  85  );     /* light red */
  colores[13] = GrAllocColor( 255, 85,  255 );     /* light magenta */
  colores[14] = GrAllocColor( 255, 255, 85  );     /* yellow */
  colores[15] = GrAllocColor( 255, 255, 255 );     /* white */

  GrEventInit();
  if( GrMouseDetect() ){
    hay_raton = 1;
    GrMouseSetInternalCursor( GR_MCUR_TYPE_ARROW,GrWhite(),GrBlack() );
    }
  muestra_raton();
  GrSoundIni();

  grf_texto = &GrFont_PC8x14;

  return 0;
}

/**************************************************************************/

int gfaz_fin( void )
{
  GrSoundEnd();
  oculta_raton();
  GrEventUnInit();
  GrSetMode( GR_default_text );
  return 0;
}

/**************************************************************************/

void gfaz_setfont( GrFont *grf )
{
  grf_texto = grf;
}

/**************************************************************************/

int lee_evento( int *valor, int espera )
{
  int i, ev;
  GrEvent gev;

  while( 1 ){
    GrEventRead(&gev);
    if (gev.type == GREV_KEY) {
      i = gev.p1;
      if( (i <= 'z') && (i >= 'a') ) i = i - 'a' + 'A';
      *valor = i;
      return EV_TECLA;
      }

    if (gev.type == GREV_MOUSE && gev.p1 == GRMOUSE_LB_PRESSED)
      if( lee_area( &ev,valor,gev.p2,gev.p3 ) )
        return ev;

    if( !espera )
      return EV_NADA;
    }
}

/**************************************************************************/

void muestra_raton( void )
{
  cuenta_puntero++;
  if( (cuenta_puntero > 0) && hay_raton ){
    GrMouseDisplayCursor();
    }
}

/************************************************************************/

void oculta_raton( void )
{
  cuenta_puntero--;
  if( (cuenta_puntero >= 0) && hay_raton ){
    GrMouseEraseCursor();
    }
}

/**************************************************************************/

void espera_soltar_raton( void )
{
  int bot, x, y;
  GrEvent gev;
  
  if( !hay_raton ) return;

  do{
    GrEventWait(&gev);
    } while( gev.type != GREV_MOUSE || gev.p1 != GRMOUSE_LB_RELEASED );
}

/************************************************************************/

void asigna_area_raton( int x, int y, struct AreaRaton *a, int n,
                        void (*f)( int,int ) )
{
  xorg = x;
  yorg = y;
  area = a;
  num_areas = n;
  fespera = f;
}

/************************************************************************/

int lee_area( int *ev, int *valor, int xi, int yi )
{
  int i, bot, x, y, nch, ncv, vb1, vb2;
  int es_tecla = 0;

  *ev = EV_NADA;
  *valor = 0;
  if( !hay_raton ) return 0;

    x = xi - xorg;
    y = yi - yorg;
    for( i=0; i<num_areas; i++ ){
      if( (x >= area[i].x1) && (x <= area[i].x2) && (y >= area[i].y1)
        && (y <= area[i].y2) ){
        if( area[i].b1 == 0 ){
          *valor = area[i].b2;
          es_tecla = 1;
          }
        else{
          nch = (x - area[i].x1) /
            ((area[i].x2 + 1 - area[i].x1) / area[i].th);
          ncv = (y - area[i].y1) /
            ((area[i].y2 + 1 - area[i].y1) / area[i].tv);
          vb1 = area[i].b1 + nch * area[i].incrh;
          vb2 = area[i].b2 + ncv * area[i].incrv;
          if( !area[i].invertir )
            *valor = (vb1 << 8) + vb2;
          else
            *valor = (vb2 << 8) + vb1;
          }
        }
      }
    if( fespera != NULL )
      fespera( es_tecla,*valor );
    else
      espera_soltar_raton();
    if( *valor != 0 ){
      if( es_tecla )
        *ev = EV_TECLA;
      else
        *ev = EV_AREA;
      }
    return 1;
}

/**************************************************************************/

void dboton( int x, int y, int an, int al,
             GrColor c, GrColor ct, char * s, int t )

//   x, y posici�n de la esquina izquierda
//   an, al ancho y alto
//   c, ct color del fondo y del texto
//   t, tipo bit 0 0=normal, 1=apretado
//           bit 1 0=no selec, 1=seleccionado
//           bit 2 0=no despl, 1=texto desplazado

{
  int pol[7][2], prof, pulsd, selec, despl;
  GrTextOption grt;
  GrLineOption glo;

  prof = (t & 0x1) ? 2 : 4;
  pulsd = (t & 0x1) ? 1 : 0;
  selec = (t & 0x2) ? 1 : 0;
  despl = (t & 0x4) ? 1 : 0;

  GrBox( x,y,x+an-1,y+al-1,K_BLACK );
  x = x + 1; y = y + 1;
  an = an - 2; al = al - 2;

  pol[0][0] = x;                 pol[0][1] = y;
  pol[1][0] = x + an - 1;        pol[1][1] = y;
  pol[2][0] = x + an - 2 - prof; pol[2][1] = y + 1 + prof;
  pol[3][0] = x + 1 + prof;      pol[3][1] = y + 1 + prof;
  pol[4][0] = x + 1 + prof;      pol[4][1] = y + al - 2 - prof;
  pol[5][0] = x;                 pol[5][1] = y + al - 1;
  pol[6][0] = pol[0][0];         pol[6][1] = pol[0][1];
  GrFilledPolygon( 7,pol,pulsd ? K_DARKGRAY : K_LIGHTGRAY );
  GrPolygon( 7,pol,K_BLACK );
  GrLine( pol[0][0],pol[0][1],pol[3][0],pol[3][1],K_BLACK );
  pol[0][0] = x + an - 1;        pol[0][1] = y + al - 1;
  pol[3][0] = x + an - 2 - prof; pol[3][1] = y + al - 2 - prof;
  pol[6][0] = pol[0][0];         pol[6][1] = pol[0][1];
  GrFilledPolygon( 7,pol,pulsd ? K_LIGHTGRAY : K_DARKGRAY );
  GrPolygon( 7,pol,K_BLACK );
  GrLine( pol[0][0],pol[0][1],pol[3][0],pol[3][1],K_BLACK );
  GrFilledBox( x+2+prof,y+2+prof,x+an-3-prof,y+al-3-prof,c );

//  grt.txo_font = &GrFont_PC8x14;
  grt.txo_font = grf_texto;
  grt.txo_fgcolor.v = ct;
  grt.txo_bgcolor.v = K_BLACK | GrOR;
  grt.txo_direct = GR_TEXT_RIGHT;
  grt.txo_xalign = GR_ALIGN_CENTER;
  grt.txo_yalign = GR_ALIGN_CENTER;
  grt.txo_chrtype = GR_BYTE_TEXT;
  if( despl )
    GrDrawString( s,strlen( s ),x+an/2+1,y+al/2+1,&grt );
  else
    GrDrawString( s,strlen( s ),x+an/2,y+al/2,&grt );

  if( selec ){
    glo.lno_color = K_BLACK;
    glo.lno_width = 1;
    glo.lno_pattlen = 2;
    glo.lno_dashpat = "\1\1";
    GrCustomBox( x+8,y+al/2-6,x+an-8,y+al/2+5,&glo );
    }
}


